#!/usr/bin/env python
# coding: utf-8

"""
``foo_api_def.yml``:

base_url: 'http://127.0.0.1:8000'
requests:
    - uri: /api/foo
      method: POST
      requests:
          - data:
              key-a: v-a
              key-b: v-b
          - data:
              key-A: v-A
              key-B: v-B
    - uri: /api/bar
      method: GET
      requests:
          - params:
              query-a: v-a
              query-b: v-b


``foo_api_test.py``:

from apibox.testing import yield_requests


def test_foo_api():
    for request_func, args in yield_requests(__file__, 'foo_api_test.yml'):
        yield request_func, args


Run using nosetest:
$ nosetests -s foo_api_test.py
"""

import requests
import requests.models
import json
import yaml
import copy
import types
import os


class RequestArguments(object):
    def __init__(self, arguments):
        if 'url' not in arguments:
            if 'base_url' in arguments:
                url = arguments['base_url'] + arguments['uri']
            else:
                url = arguments['uri']
            arguments['url'] = url
        self.arguments = arguments

    @property
    def data_str(self):
        data = self.arguments.get('data')
        if data:
            return json.dumps(data, ensure_ascii=False).encode('utf8')
        return ''

    @property
    def params_str(self):
        params = self.arguments.get('params')
        if params:
            return requests.models.RequestEncodingMixin._encode_params(params)
        return ''

    def __str__(self):
        d = dict(self.arguments)
        if 'data' in self.arguments:
            short = self.data_str[:50]
        elif 'params' in self.arguments:
            short = self.params_str[:50]
        else:
            short = ''
        d['params_or_data_short'] = short
        return '{method} {url} {params_or_data_short}'.format(**d)

    def __repr__(self):
        return str(self)


def do_request(req_args):
    args = req_args.arguments
    func = getattr(requests, args['method'].lower())
    kwargs = {i: args[i] for i in args
              if i in ['params', 'data', 'headers', 'cookies']}
    resp = func(args['url'], **kwargs)
    print resp.status_code, resp.content
    assert resp.status_code == 200


def yield_args(args, df):
    _args = copy.copy(df)
    if 'requests' in _args:
        reqs = _args.pop('requests')
        args.update(_args)
        for child_df in reqs:
            yielded = yield_args(copy.copy(args), child_df)
            if isinstance(yielded, types.GeneratorType):
                for i in yielded:
                    yield i
            else:
                yield yielded
    else:
        args.update(_args)
        yield args


def yield_requests(current_path, filename):
    filepath = os.path.join(
        os.path.dirname(current_path), filename)
    with open(filepath, 'r') as f:
        requests_def = yaml.load(f.read())

    for args in yield_args({}, requests_def):
        yield do_request, RequestArguments(args)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from urllib import urlencode
import requests
from .log import logger


class APIBase(object):
    """
    Usage:

    >>> class PinboardAPI(APIBase):
    ...     base_url = 'https://api.pinboard.in/v1'
    ...     uris = {
    ...         '/posts/all': {
    ...             'method': 'GET'
    ...         },
    ...         '/posts/recent': {
    ...             'method': 'GET'
    ...         },
    ...     }
    >>> api = API(token)
    >>> body = api.posts.all(raw=True)
    """
    base_url = None
    uris = None
    default_content_type = 'text'  # or 'json'
    token_config = None
    #token_config = {
    #    'in': 'params',  # or 'headers'
    #    'key': None
    #}

    def __init__(self, token=None):
        # Check attributes
        if not self.base_url:
            raise ValueError('`base_url` is required')
        if not self.uris:
            raise ValueError('`uris` is required')

        if self.token_config:
            if not self.token_config.get('key'):
                raise ValueError('`key` in `token_config` is required')

            # Set token
            if token:
                self.token_config['value'] = token

            if not self.token_config.get('value'):
                raise ValueError('if `token_config` is set, `token` must be set or passed')
        else:
            if token:
                raise ValueError('`token_config` must be set to pass `token` into it')

    def __getattr__(self, name):
        return ResourcePath(self, name)

    def get_url(self, uri, params=None):
        url = self.base_url + uri
        if params:
            url += '?' + urlencode(params)
        return url

    def get_headers(self):
        headers = {}
        if self.token_is_in('headers'):
            token_key = self.token_config['key']
            headers[token_key] = self.token_config['value']
        return headers

    def token_is_in(self, position):
        if not self.token_config:
            return False
        return position == self.token_config['in']

    def make_req(self, uri, *args, **kwargs):
        """
        kwargs:
        - params

        uri supported options:
        {
            'method': 'GET',
            'content_type': 'text',
            'default_params': {'a': 1},
        }
        """
        try:
            options = self.uris[uri]
        except KeyError:
            raise Exception('URI %s is not in uris' % uri)
        method = options['method'].lower()

        # handle `content_type`
        content_type = options.get('content_type', self.default_content_type)

        # handle `default_params`
        params = options.get('default_params', {})
        passing_params = kwargs.get('params', {})
        params.update(passing_params)

        # handle token
        if self.token_is_in('params'):
            token_key = self.token_config['key']
            params[token_key] = self.token_config['value']

        url = self.get_url(uri, params)
        headers = self.get_headers()
        requester = getattr(requests, method)

        logger.info('[REQUEST] %s %s', requester.__name__, uri)
        resp = requester(url, headers=headers)

        if content_type == 'json':
            return resp.json()
        else:  # == 'text'
            return resp.content


class ResourcePath(object):
    def __init__(self, api, name, parent=None):
        self._api = api
        self._name = name
        self._parent = parent

    def __getattr__(self, name):
        return ResourcePath(self._api, name, self)

    def __call__(self, *args, **kwargs):
        return self._api.make_req(self.get_path(), *args, **kwargs)

    def get_path(self):
        names = []

        rp = self
        while rp is not None:
            names.insert(0, rp._name)
            rp = rp._parent

        return '/' + '/'.join(names)

    def __str__(self):
        return '<ResourcePath %s>' % self.get_path()



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup


# Use semantic versioning: MAJOR.MINOR.PATCH
version = '0.1.0'


def get_requires():
    with open('requirements.txt', 'r') as f:
        requires = [i for i in map(lambda x: x.strip(), f.readlines()) if i]
    return requires


def get_long_description():
    try:
        with open('README.md', 'r') as f:
            return f.read()
    except IOError:
        return ''


setup(
    # license='License :: OSI Approved :: MIT License',
    name='apibox',
    version=version,
    author='reorx',
    author_email='novoreorx@gmail.com',
    description='Building blocks for HTTP API development',
    url='https://github.com/reorx/apibox',
    long_description=get_long_description(),
    packages=[
        'apibox',
    ],
    # Or use (make sure find_packages is imported from setuptools):
    # packages=find_packages()
    install_requires=get_requires(),
    # package_data={}
    # entry_points={'console_scripts': ['foo = package.module:main_func']}
)
